﻿<?php
// клас за създаване на котки, но с вграден контрол на раждаемостта.
class Cat extends Animal {
    // максимален брой на котките, които могат да бъдат създадени
    const MAX_CATS = 2;
    // това е броят създадени котки.
    public static function getTotalCats() {
        return self::$totalCats;
    }
    private static $totalCats = 0;
    
	// в PHP не можем да декларираме конструктора на класа наследник с по-ограничена
	// видимост от конструктора на базовия клас, което пък прецаква идеята в нашия
	// случай инстанции на Cat да бъдат създавани само чрез статичния метод createCat()
    protected function __construct($name, $isAsleep = null) {
		parent::__construct($name, $isAsleep);
    }
    
    // тъй като конструкторът може да бъде извикан само в рамките на този клас,
    // създаването на обект от клас Cat може да стане само чрез следните два
    // метода, в които осъществяваме "контрола на раждаемостта". Това е Cat Factory.
    public static function createCat($name, $isAsleep = null) {
        if (self::$totalCats + 1 > self::MAX_CATS)
            throw new AnimalException("Твърде много котки!");
        
        $cat = new Cat($name, $isAsleep);
        
        self::$totalCats++;
        
        return $cat;
    }
    
    // имплементираме метода Sound дефиниран в ISoundable интерфейса
    // и в последствие деклариран като абстрактен в класа Animal
    public function sound() {
		print("Мяу!\n");
    }
}
?>